@@ -45,6 +45,7 @@ class AdyenResult(object):
 
 
 class AdyenClient(object):
+    IDEMPOTENCY_HEADER_NAME = 'Idempotency-Key'
     """A requesting client that interacts with Adyen. This class holds the
     adyen logic of Adyen HTTP API communication. This is the object that can
     maintain its own username, password, merchant_account, hmac and skin_code.
@@ -64,15 +65,32 @@ class AdyenClient(object):
         skin_code (str, optional): skin_code to place directory_lookup requests
             and generate hpp signatures with.
         hmac (str, optional): Hmac key that is used for signature calculation.
+        http_timeout (int, optional): The timeout in seconds for HTTP calls,
+            default 30.
     """
 
-    def __init__(self, username=None, password=None, xapikey=None,
-                 review_payout_username=None, review_payout_password=None,
-                 store_payout_username=None, store_payout_password=None,
-                 platform="test", merchant_account=None,
-                 merchant_specific_url=None, skin_code=None,
-                 hmac=None, app_name="",
-                 http_force=None, live_endpoint_prefix=None):
+    def __init__(
+        self,
+        username=None,
+        password=None,
+        xapikey=None,
+        review_payout_username=None,
+        review_payout_password=None,
+        store_payout_username=None, store_payout_password=None,
+        platform="test", merchant_account=None,
+        merchant_specific_url=None, skin_code=None,
+        hmac=None,
+        http_force=None,
+        live_endpoint_prefix=None,
+        http_timeout=30,
+        api_bin_lookup_version=None,
+        api_checkout_utility_version=None,
+        api_checkout_version=None,
+        api_payment_version=None,
+        api_payout_version=None,
+        api_recurring_version=None,
+        api_terminal_version=None,
+    ):
         self.username = username
         self.password = password
         self.xapikey = xapikey
@@ -86,15 +104,21 @@ class AdyenClient(object):
         self.merchant_account = merchant_account
         self.skin_code = skin_code
         self.psp_list = []
-        self.app_name = app_name
         self.LIB_VERSION = settings.LIB_VERSION
         self.USER_AGENT_SUFFIX = settings.LIB_NAME + "/"
         self.http_init = False
         self.http_force = http_force
         self.live_endpoint_prefix = live_endpoint_prefix
+        self.http_timeout = http_timeout
+        self.api_bin_lookup_version = api_bin_lookup_version or settings.API_BIN_LOOKUP_VERSION
+        self.api_checkout_utility_version = api_checkout_utility_version or settings.API_CHECKOUT_UTILITY_VERSION
+        self.api_checkout_version = api_checkout_version or settings.API_CHECKOUT_VERSION
+        self.api_payment_version = api_payment_version or settings.API_PAYMENT_VERSION
+        self.api_payout_version = api_payout_version or settings.API_PAYOUT_VERSION
+        self.api_recurring_version = api_recurring_version or settings.API_RECURRING_VERSION
+        self.api_terminal_version = api_terminal_version or settings.API_TERMINAL_VERSION
 
-    @staticmethod
-    def _determine_api_url(platform, service, action):
+    def _determine_api_url(self, platform, service, action):
         """This returns the Adyen API endpoint based on the provided platform,
         service and action.
 
@@ -103,13 +127,24 @@ class AdyenClient(object):
             service (str): API service to place request through.
             action (str): the API action to perform.
         """
-        base_uri = settings.BASE_PAL_URL.format(platform)
-        if service == "Recurring":
-            api_version = settings.API_RECURRING_VERSION
-        elif service == "Payout":
-            api_version = settings.API_PAYOUT_VERSION
+        if platform == "live" and self.live_endpoint_prefix:
+            base_uri = settings.PAL_LIVE_ENDPOINT_URL_TEMPLATE.format(
+                self.live_endpoint_prefix
+            )
         else:
-            api_version = settings.API_PAYMENT_VERSION
+            base_uri = settings.BASE_PAL_URL.format(platform)
+
+        if service == "Recurring":
+            api_version = self.api_recurring_version
+        elif service == "Payout":
+            api_version = self.api_payout_version
+        elif service == "BinLookup":
+            api_version = self.api_bin_lookup_version
+        elif service == "terminal":
+            base_uri = settings.BASE_TERMINAL_URL.format(platform)
+            api_version = self.api_terminal_version
+        else:
+            api_version = self.api_payment_version
         return '/'.join([base_uri, service, api_version, action])
 
     @staticmethod
@@ -127,15 +162,16 @@ class AdyenClient(object):
         result = '/'.join([base_uri, service])
         return result
 
-    def _determine_checkout_url(self, platform, action):
+    def _determine_checkout_url(self, platform, action, path_param=None):
         """This returns the Adyen API endpoint based on the provided platform,
         service and action.
 
         Args:
             platform (str): Adyen platform, ie 'live' or 'test'.
             action (str): the API action to perform.
+            path_param Optional[(str)]: a generic id that can be used to modify a payment e.g. paymentPspReference.
         """
-        api_version = settings.API_CHECKOUT_VERSION
+        api_version = self.api_checkout_version
         if platform == "test":
             base_uri = settings.ENDPOINT_CHECKOUT_TEST
         elif self.live_endpoint_prefix is not None and platform == "live":
@@ -153,8 +189,22 @@ class AdyenClient(object):
             action = "payments/details"
         if action == "paymentsResult":
             action = "payments/result"
+        if action == "cancels":
+            action = "cancels"
+        if action == "paymentsCancelsWithReference":
+            action = f"payments/{path_param}/cancels"
+        if action == "paymentsCapture":
+            action = f"payments/{path_param}/captures"
+        if action == "paymentsReversals":
+            action = f"payments/{path_param}/reversals"
+        if action == "paymentsRefunds":
+            action = f"payments/{path_param}/refunds"
         if action == "originKeys":
-            api_version = settings.API_CHECKOUT_UTILITY_VERSION
+            api_version = self.api_checkout_utility_version
+        if action == "paymentMethodsBalance":
+            action = "paymentMethods/balance"
+        if action == "ordersCancel":
+            action = "orders/cancel"
 
         return '/'.join([base_uri, api_version, action])
 
@@ -198,32 +248,36 @@ class AdyenClient(object):
         'Adyen.store_payout_password = 'Your payout password'"""
         raise AdyenInvalidRequestError(errorstring)
 
-    def call_api(self, request_data, service, action, idempotency=False,
-                 **kwargs):
+    def call_api(
+        self,
+        request_data,
+        service,
+        action,
+        idempotency_key=None,
+        **kwargs
+    ):
         """This will call the adyen api. username, password, merchant_account,
         and platform are pulled from root module level and or self object.
         AdyenResult will be returned on 200 response. Otherwise, an exception
         is raised.
 
         Args:
+            idempotency_key: https://docs.adyen.com/development-resources
+            /api-idempotency
             request_data (dict): The dictionary of the request to place. This
                 should be in the structure of the Adyen API.
-                https://docs.adyen.com/manuals/api-manual
+                https://docs.adyen.com/api-explorer
             service (str): This is the API service to be called.
             action (str): The specific action of the API service to be called
             idempotency (bool, optional): Whether the transaction should be
                 processed idempotently.
-                https://docs.adyen.com/manuals/api-manual#apiidempotency
+                https://docs.adyen.com/development-resources/api-idempotency
         Returns:
             AdyenResult: The AdyenResult is returned when a request was
-                succesful.
+                successful.
         """
         if not self.http_init:
-            self.http_client = HTTPClient(self.app_name,
-                                          self.USER_AGENT_SUFFIX,
-                                          self.LIB_VERSION,
-                                          self.http_force)
-            self.http_init = True
+            self._init_http_client()
 
         # username at self object has highest priority. fallback to root module
         # and ensure that it is set.
@@ -313,8 +367,8 @@ class AdyenClient(object):
         # Adyen requires this header to be set and uses the combination of
         # merchant account and merchant reference to determine uniqueness.
         headers = {}
-        if idempotency:
-            headers['Pragma'] = 'process-retry'
+        if idempotency_key:
+            headers[self.IDEMPOTENCY_HEADER_NAME] = idempotency_key
 
         url = self._determine_api_url(platform, service, action)
 
@@ -336,6 +390,15 @@ class AdyenClient(object):
 
         return adyen_result
 
+    def _init_http_client(self):
+        self.http_client = HTTPClient(
+            user_agent_suffix=self.USER_AGENT_SUFFIX,
+            lib_version=self.LIB_VERSION,
+            force_request=self.http_force,
+            timeout=self.http_timeout,
+        )
+        self.http_init = True
+
     def call_hpp(self, message, action, hmac_key="", **kwargs):
         """This will call the adyen hpp. hmac_key and platform are pulled from
         root module level and or self object.
@@ -345,12 +408,9 @@ class AdyenClient(object):
         Args:
             request_data (dict): The dictionary of the request to place. This
                 should be in the structure of the Adyen API.
-                https://docs.adyen.com/manuals/api-manual
+                https://docs.adyen.com/online-payments/classic-integrations/hosted-payment-pages/hosted-payment-pages-api
             service (str): This is the API service to be called.
             action (str): The specific action of the API service to be called
-            idempotency (bool, optional): Whether the transaction should be
-                processed idempotently.
-                https://docs.adyen.com/manuals/api-manual#apiidempotency
         Returns:
             AdyenResult: The AdyenResult is returned when a request was
                 succesful.
@@ -358,11 +418,7 @@ class AdyenClient(object):
                 :param hmac_key:
         """
         if not self.http_init:
-            self.http_client = HTTPClient(self.app_name,
-                                          self.USER_AGENT_SUFFIX,
-                                          self.LIB_VERSION,
-                                          self.http_force)
-            self.http_init = True
+            self._init_http_client()
 
         # hmac provided in function has highest priority. fallback to self then
         # root module and ensure that it is set.
@@ -409,25 +465,25 @@ class AdyenClient(object):
                                              status_code, headers, message)
         return adyen_result
 
-    def call_checkout_api(self, request_data, action, **kwargs):
+    def call_checkout_api(self, request_data, action, idempotency_key=None, path_param=None,
+                          **kwargs):
         """This will call the checkout adyen api. xapi key merchant_account,
         and platform are pulled from root module level and or self object.
         AdyenResult will be returned on 200 response. Otherwise, an exception
         is raised.
 
         Args:
+            idempotency_key: https://docs.adyen.com/development-resources
+            /api-idempotency
             request_data (dict): The dictionary of the request to place. This
                 should be in the structure of the Adyen API.
-                https://docs.adyen.com/developers/checkout/api-integration
+                https://docs.adyen.com/api-explorer/#/CheckoutService
             service (str): This is the API service to be called.
             action (str): The specific action of the API service to be called
+            path_param (str): This is used to pass the id or referenceID to the API sercie
         """
         if not self.http_init:
-            self.http_client = HTTPClient(self.app_name,
-                                          self.USER_AGENT_SUFFIX,
-                                          self.LIB_VERSION,
-                                          self.http_force)
-            self.http_init = True
+            self._init_http_client()
 
         # xapi at self object has highest priority. fallback to root module
         # and ensure that it is set.
@@ -460,25 +516,38 @@ class AdyenClient(object):
         if not request_data.get('merchantAccount'):
             request_data['merchantAccount'] = self.merchant_account
 
-        if 'applicationInfo' in request_data:
-            request_data['applicationInfo'].update({
-                "adyenLibrary": {
-                    "name": settings.LIB_NAME,
-                    "version": settings.LIB_VERSION
+        with_app_info = [
+            "authorise",
+            "authorise3d",
+            "authorise3ds2",
+            "payments",
+            "paymentSession",
+            "paymentLinks",
+            "paymentMethodsBalance",
+            "sessions"
+        ]
+
+        if action in with_app_info:
+            if 'applicationInfo' in request_data:
+                request_data['applicationInfo'].update({
+                    "adyenLibrary": {
+                        "name": settings.LIB_NAME,
+                        "version": settings.LIB_VERSION
+                    }
+                })
+            else:
+                request_data['applicationInfo'] = {
+                    "adyenLibrary": {
+                        "name": settings.LIB_NAME,
+                        "version": settings.LIB_VERSION
+                    }
                 }
-            })
-        else:
-            request_data['applicationInfo'] = {
-                "adyenLibrary": {
-                    "name": settings.LIB_NAME,
-                    "version": settings.LIB_VERSION
-                }
-            }
         # Adyen requires this header to be set and uses the combination of
         # merchant account and merchant reference to determine uniqueness.
         headers = {}
-
-        url = self._determine_checkout_url(platform, action)
+        if idempotency_key:
+            headers[self.IDEMPOTENCY_HEADER_NAME] = idempotency_key
+        url = self._determine_checkout_url(platform, action, path_param)
 
         raw_response, raw_request, status_code, headers = \
             self.http_client.request(url, json=request_data,
@@ -493,13 +562,8 @@ class AdyenClient(object):
         return adyen_result
 
     def hpp_payment(self, request_data, action, hmac_key="", **kwargs):
-
         if not self.http_init:
-            self.http_client = HTTPClient(self.app_name,
-                                          self.USER_AGENT_SUFFIX,
-                                          self.LIB_VERSION,
-                                          self.http_force)
-            self.http_init = True
+            self._init_http_client()
 
         platform = self.platform
         if not isinstance(platform, str):
@@ -547,7 +611,7 @@ class AdyenClient(object):
         Returns:
             AdyenResult: Result object if successful.
         """
-        if status_code != 200:
+        if (status_code != 200 and status_code != 201):
             response = {}
             # If the result can't be parsed into json, most likely is raw html.
             # Some response are neither json or raw html, handle them here:
@@ -582,7 +646,7 @@ class AdyenClient(object):
         else:
             try:
                 response = json_lib.loads(raw_response)
-                psp = headers.get('pspReference', response.get('pspReference'))
+                psp = self._get_psp(response, headers)
                 return AdyenResult(message=response, status_code=status_code,
                                    psp=psp, raw_request=raw_request,
                                    raw_response=raw_response)
@@ -730,3 +794,11 @@ class AdyenClient(object):
         match_obj = re.search(r'>Error:\s*(.*?)<br', html)
         if match_obj:
             return match_obj.group(1)
+
+    @staticmethod
+    def _get_psp(response, header):
+        psp_ref = response.get('pspReference')
+        if psp_ref == "":
+            return header.get('pspReference')
+        else:
+            return psp_ref
